<?php
#################################################
//	RiddleMeThis PHP Runtime
//	
//	Written by and copyright (c) 2003-2007 Tonio Loewald
//	tonio@loewald.com 
//	
//	This program is licensed to anyone under the
//	GNU Lesser General Public License
//	http://www.opensource.org/licenses/lgpl-license.php
//
//  Version 1.0.11 July 2008
//  * Better feedback if Data directory is not correctly set up
//  * Automatically places blank index.html file in Data directory if none found
//
//	Version 1.0.10c April 2008
//	* Fix to video cropping issue
//	* Fix IE rendering problems
//	* Restore attractiveness on Firefox and Safari
//
//	Version 1.0.9e February, 2008
//	* Added support for variable insertion into question text
//
//	Version 1.0.9d August, 2007
//	* Cleaned up debug code (doesn't mess up layout now)
//
//	Version 1.0.9c August, 2007
//	* Extensive formatting improvements
//	* CSS used for most formatting, instead of tables
//	* Various minor bug fixes
//
//	Version 1.0.9b August, 2007
//	* Implement section background pictures
//
//	Version 1.0.9a July, 2007
//  * Quick hack to accept data from UH's experiment tracking system
//	If you pass a $_GET parameter called "survey_code" when you start the form,
//	it will be recorded in the data file. This is for integrating with other backend systems.
//
//	Version 1.0.8 February, 2007
//	* Fixed Bug in Go Back functionality (it works again!)
//	* Fixed several bugs in implementation of "Pick All That Apply" questions
//
//	Version 1.0.7b December, 2006
//	* Implemented support for section timers
//	* Fixed bug in Check All That Apply questions
//	* Fixed bug -- hardwired reference to 'Continue' button
//
//	Version 1.0.6 January, 2005
//	* Implemented show range values
//	* Numerous bugs that did not manifest in PHP4 but did manifest in PHP5 have been fixed
//	  (This is not a fault in PHP5, more a fault in PHP4 and my code!)
//	* Numerous issues that would raise warning flags (but were not, in fact, bugs)
//	  have also been addressed.
//
//	Version 1.0.5 September 26, 2004
//	* Implemented Check All That Apply questions
//
//	Version 1.0.4 April 20, 2004
//  * Corrected bug in session id generation (months vs minutes)
//
//	Version 1.0.3 October 30, 2003
//	* Corrected bug that was causing warnings and misbehavior
//	  with some PHP installations.
//
//	Version 1.0.2 September 25, 2003
//	* Corrected bug relating to column names containing SPACES.
//
//	Version 1.0.1 September 24, 2003
//	* Corrected bug in headings which did not remove "answer all
//	  questions" text when it's not necessary.
//	* The prompt (that the user answer all questions, or whatever
//	  you choose to change it to) now only appears if there is at 
//	  least one unanswered question on the page.
//	* Modified redirection to use Javascript first, META refresh 
//	  second, and finally allow the user to manually click the
//	  link.
//	  Javascript added because IE does not properly implement 
//	  META refresh.
//
//	Version 1.0 September 16 2003
//	* Changed licensing to LGPL
//
//	Version 1.0B11 September 16 2003
//	* Fixed bug in conditional sections (they should work now!)
//	  Many many thanks to Carlos Navarette for his help figuring
//	  this bug out.
//	* Debug settings are now selected from RiddleMeThis.
//	* Other settings can also be modified from RiddleMeThis.
//
//	Version 1.0B10
//	* Fix: No longer assumes the file will be named "form.rmt"
//	  rather than "Form.rmt" for case-sensitive servers
//	* Fix: no longer assumes form.rmt will be smaller than 32kB
//	  (yes, ouch!)
//	
//	Version 1.0B9 June 28 2003
//
//	* Implemented random variable assignment
//	* Implemented conditional sections
//	* Switched calls to rand() to mt_rand()
//	  based on recommendations in PHP online docs.
//	* Fixed display of quotation marks in questions.
//
//	Version 1.0B8 May 6 2003
//	* More stuff put into constants for easier
//	  end-user modification.
//	* Heading message now changes when form is
//	  complete.
//
//	Version 1.0B7
//	* Decode question text for all items not just
//    plain text items. [Fix]
//	* Corrected case error in DATA_PATH (make sure
//	  your data directory is named "Data" or modify
//	  the DATA_PATH constant below).
//	Change as of 1.0:
//	  Note that DATA_PATH is now set inside RiddleMeThis.
//
//	Version 1.0B6
//	* Most questionnaire preferences implemented!
//	- ForwardEnabledOnCompletion
//  - BackButtonEnabled
//  - ShowSection
//  - ShowProgress
//	* Better feedback for unanswered questions

//	* Changed promotional links to target _blank
//	* Replaced hard-wired paths with global constants
//	* No longer spam PHP comments in web page!
//	* Minor efforts toward PHP beautification :)
//	
//	Version 1.0B5 March 27 2003
//	* Enforce filling in of forms
//	* Use constants to control debug feedback
#################################################
##               PHP Utilities
#################################################
	function SetFormConstant($settings, $itsname, $defaultvalue) {
		$setting = $settings[$itsname];
		if( is_string($setting) ) {
			switch( $setting ) {
				case '':
					define($itsname, $defaultvalue);
					break;
				case 'false':
					define($itsname, false);
					break;
				case 'true':
					define($itsname, true);
					break;
				default:
					define($itsname, $setting);
					break;
			}
		} else {
			define($itsname, $defaultvalue);
		}
	}
	
	function LoadSettings($form) {
		$prefs = $form['prefs'];
		// Costmetic Constants
		define('COLOR_BACKGROUND', $prefs['ColorBackground']);
		define('COLOR_HEADER', $prefs['ColorHeader']);
		define('COLOR_TEXT', $prefs['ColorText']);
		define('COLOR_ALERT', $prefs['ColorAlert']);
		define('FONT_OPTIONS', $prefs['FontOptions']);
		define('FONT_SIZE', $prefs['FontSize']);
		
		$settings = stringToArray($prefs['OnlineSettings'], chr(4), chr(5));
		
		// DEBUG CONSTANTS
		SetFormConstant($settings, 'DEBUG', false);
		// To show $results and $_POST
		SetFormConstant($settings, 'DEBUG_RESULTS', false);
		// To show form preferences
		SetFormConstant($settings, 'DEBUG_FORMPREFS', false);
		// To show server data
		SetFormConstant($settings, 'DEBUG_SERVER', false);
		// To show the answer verification details
		SetFormConstant($settings, 'DEBUG_VERIFY', false);
		// To see a breakdown of the form as it loads
		SetFormConstant($settings, 'DEBUG_ITEMS', false);
		// To show random variable settings
		SetFormConstant($settings, 'DEBUG_RANDOMVARIABLES', false);
		// To show conditional sections at work
		SetFormConstant($settings, 'DEBUG_CONDITIONALSECTIONS', false);
		
		// PATH CONSTANTS
		SetFormConstant($settings, 'DATA_PATH', 'Data/');
		SetFormConstant($settings, 'VIDEO_PATH', 'Videos/');
		SetFormConstant($settings, 'IMAGE_PATH', 'Images/');
		
		// OTHER CONSTANTS
		// Heading message when user completes form
		SetFormConstant($settings, 'HEADING_DONE', 'Form Completed');
		// Heading message shown when user must answer all questions
		SetFormConstant($settings, 'HEADING_ANSWERALL', 'Please answer all questions before proceeding...');
		// Heading message shown when user must answer all questions and tries to proceed with some not answered
		SetFormConstant($settings, 'HEADING_ANSWERHILITED', 'Please answer <b>hilighted</b> questions before proceeding...');
		// Message displayed when user completes form
		SetFormConstant($settings, 'MESSAGE_THANKS', 'Thank you for completing this form.');
		// Window title
		SetFormConstant($settings, 'FORM_NAME', 'RiddleMeThis Runtime');
		// Footer
		SetFormConstant($settings, 'FORM_FOOTER', 'This form was created by <a href="http://www.loewald.com/RiddleMeThis/" target="_blank">RiddleMeThis</a> Runtime, &copy;2002-8 <a href="http://www.loewald.com/" target="_blank">Tonio Loewald</a>.');
		// Text caption of continue button
		SetFormConstant($settings, 'BUTTON_CONTINUE_NAME', 'Continue');
		// Text caption of back button
		SetFormConstant($settings, 'BUTTON_BACK_NAME', 'Back');
		// Redirect on completion
		SetFormConstant($settings, 'REDIRECT_ON_COMPLETION', false);
		SetFormConstant($settings, 'REDIRECT_ON_COMPLETION_URL', 'thankyou.html');
		SetFormConstant($settings, 'REDIRECT_ON_COMPLETION_MESSAGE', 'Please click here if your browser does not proceed within a few seconds.');
	}

	################################
	## delivers raw POST parameters
	################################
	function rawp($name) {
		return ini_get('magic_quotes_gpc')
			? stripslashes($_POST[$name])
			: $_POST[$name];
	}
		
	####################################################
	## Simple text file i/o routines
	## NB: permissions need to be set or they will fail
	####################################################
	
	function savetext($filename, $filetext) {
		$file = fopen($filename, 'w');
		if ( $file ) {
			$err = fwrite($file, $filetext);
			$err = fclose($file);
		}
	}
	
	function loadtext($filename) {
		if( file_exists( $filename )) {
			$file = fopen($filename, 'r');
			if ( $file ) {
				$filetext = fread($file, 1000000);
				$err = fclose($file);
				$filetext = $filetext;
				return $filetext;
			} else {
				return '';
			}
		} else { 
			return '';
		}
	}
	
	function conditionalString($condition, $string1, $string2='') {
		if($condition)
			return $string1;
		else
			return $string2;
	}
	
	################################################
	## Displays a table of POST variables submitted
	################################################
	function listvars() {
		if( $_SERVER['REQUEST_METHOD'] == 'POST' ) {
			debugArray( $_POST );
		}
	}
	
	####################################################
	## Append newly submitted responses to session file
	####################################################
	function UpdateSession($SessionID) {
		global $results;
		// Create associative array from session file
		$SessionFilePath = DATA_PATH . $SessionID . '.txt';
		$existing = loadtext($SessionFilePath);
		$lines = split(chr(13), $existing, 3);
		if ( count($lines) == 3 ) {
			$ip = $lines[2];
			if( $ip == $_SERVER['REMOTE_ADDR']) {
				$columns = split(chr(9), $lines[0]);
				$values = split(chr(9), $lines[1]);
				
				// echoP('Columns', 'h2');
				// debugArray($columns);
				
				// echoP('Values', 'h2');
				// debugArray($values);
				
				for( $i = 0; $i < count($columns); $i++) {
					if ($columns[$i] != '') {
						$results[$columns[$i]] = $values[$i];
					}
				}
			}
		}
		
		// Add/Modify array based on new data
		foreach ($_POST as $prop => $value) {
			if( substr($prop, 0, 4) == "COL_" ) {
				$results[substr($prop, 4)] = EncodeAnswer($value);
			}
		}
		// echoP( 'Results', 'h2' );
		// debugArray( $results );
		
	}
	
	function SaveSession($SessionID) {
		global $results;
		
		// echoP( 'Saving session...', 'h3' );
		// debugArray( $results );
		
		// Capture GET variables
		if( isset($_GET["survey_code"]) ){
			$results["survey_code"] = $_GET["survey_code"];
		}
		
		$columns = '';
		$values = '';
		foreach($results as $prop => $value) {
			$columns = $columns . $prop . chr(9);
			$values = $values . $value . chr(9);
		}
		
		// chmod(DATA_PATH, 0777);
		if (! file_exists( DATA_PATH ) || is_file( DATA_PATH ) ){
		    die('<h1>Fatal Error</h1><p>The Data path (' . DATA_PATH . ') is missing (or not a directory).</p>');
		}
		if (! is_writeable( DATA_PATH ) ){
		    die('<h1>Fatal Error</h1><p>The Data path (' . DATA_PATH . ') is not writeable. You need to set its permissions correctly (Try 755. If this error still occurs, use 777).</p>');
		}
		if (! file_exists( DATA_PATH . 'index.html' )){
		    savetext( DATA_PATH . 'index.html', '' );
		}
		if ( Done() ) {
			$SessionFilePath = DATA_PATH . $SessionID . '_completed.txt';
		} else {
			$SessionFilePath = DATA_PATH . $SessionID . '.txt';
		}
		savetext($SessionFilePath, $columns . chr(13) . $values . chr(13) . $_SERVER['REMOTE_ADDR']);
		// Save to second location?
		// Consider deleting temp file.
	}
	
	#####################################################
	## Decode / Encode Slashes (RMT compatibility)
	#####################################################
	function DecodeQuestion($s) {
		global $results;
		
		// Substitute variables where appropriate.
		$parts = split('<', $s);
		$t = $parts[0];
		for($i = 1; $i < count($parts); $i++){
			$part = $parts[$i];
			$pieces = split('>', $part, 2);
			if(count($pieces) == 1){
				$t .= $part;
			} else {
				$t .= $results[$pieces[0]] . $pieces[1];
			}
		}
		$t = str_replace('\\n', '<br>', $t);
		$t = str_replace('\\q', '&quot;', $t);
		return $t;
	}
	
	function DecodeAnswer($s) {
		$t = str_replace('\\n', chr(13), $s);
		return $t;
	}
	
	function EncodeAnswer($s) {
		$t = str_replace(chr(13), '\\n', $s);
		return $t;
	}
	
	#####################################################
	## echos a string surrounded by standard paired tags
	#####################################################
	function echoP($msg, $tag="P") {
		echo "<" . $tag . ">\r";
		echo "\t" . $msg . "\r";
		echo "</" . $tag . ">\r";
	}
	
	function echoR($msg) {
		echo $msg . "\r";
	}
	
	$debugOutput = '';
	
	function debugP($msg, $tag="P") {
		global $debugOutput;
		$debugOutput .= "<" . $tag . ">\r";
		$debugOutput .= "\t" . $msg . "\r";
		$debugOutput .= "</" . $tag . ">\r";
	}
	
	function debugR($msg) {
		global $debugOutput;
		$debugOutput .=  $msg . "\r";
	}
	
	#######################################
	## Displays an array as a simple table
	#######################################
	function debugArray($theArray) {
		if ( count($theArray) ) {	
			debugR( '<table padding=2 border=1><tr>' );
			debugP( 'Property', 'th' );
			debugP( 'Value', 'th' );
			debugR( '</tr>' );
			foreach ($theArray as $prop => $value) {
				debugR('<tr>');
				debugP( $prop, "td" );
				if (is_array($value)) {
					debugR('<td>');
					debugArray( $value );
					debugR('</td>');
				} else {
					debugP( $value, "td" );
				}
				debugR( '</tr>' );
			}
			debugR( '</table>' );
		}
	}
	
	##########################################
	## Break string into an associative array
	##########################################
	function stringToArray($s, $propDelim='=', $itemDelim='\r') {
		$items = split($itemDelim, $s);
		$a = array();
		for($i = 0; $i < count($items); $i++) {
			// Added limit to allow for the conditions array (which has three columns)
			$splitItem = split($propDelim, $items[$i], 2);
			if((count($splitItem) > 1) && ($splitItem[0] != '')) {
				$a[$splitItem[0]] = $splitItem[1];
			}
		}
		return $a;
	}
	
	########################################################################
	## Load Form Specification
	## This function loads a RiddleMeThis form document into two
	## associative arrays...
	## $formPrefs contains the global form settings
	## $sections contains the sections (each section contains its own items
	########################################################################
	function loadForm() {
		// 1.0B10 fix for case-sensitive servers
		if(file_exists('form.rmt')) {
			$formSpec = split(chr(13), loadtext( 'form.rmt' ));
		}
		elseif(file_exists('Form.rmt')) {
			$formSpec = split(chr(13), loadtext( 'Form.rmt' ));
		}
		else {
			die('Could not find form specification file "form.rmt".');
		}
		$sections = array();
		$formPrefs = array();
	
		for($i = 0; $i < count($formSpec); $i++) {
			// Changed to prevent use of '=' in conditions from breaking
			// this line of code:
			$theItem = split('=', $formSpec[$i], 2);
			if(count($theItem)>1){
				$itemType = $theItem[0];
				$itemSpec = stringToArray($theItem[1], chr(7), chr(10));
				switch($itemType) {
				case 'FORM':
					$formPrefs = $itemSpec;
					break;
				case 'SECTION':
					$itemSpec['items'] = array();
					$sections[] = $itemSpec;
					break;
				case 'ITEM':
					if(count($sections)) {
						$sections[count($sections)-1]['items'][] = $itemSpec;
					} else {
						die('Fatal error in form.rmt. ITEM encountered before first SECTION.');
					}
					break;
				}
			}
		}
		
		return array('prefs' => $formPrefs, 'sections' => $sections);
	}
	##################
	## Form Complete?
	##################
	function done( $offset=0 ) {
		global $CurrentPageNumber;
		if ($CurrentPageNumber + $offset > sectioncount())
			return true;
		else
			return false;
	}
	
	function sectioncount() {
		global $form;
		return count($form['sections']);
	}
	
	#################
	## VerifyAnswers
	#################
	function VerifyAnswers($page) {
		global $form;
		global $results;
		global $UnansweredQuestions;
		
		if($page < 1) {
			$verified = 1;
		} else {
			if( $form['prefs']['ForwardEnabledOnCompletion'] != 'TRUE' ){
				return 1;
			}
			$verified = 1;
			$section = $form['sections'][$page - 1];
			$items = $section['items'];
			foreach($items as $item) {
				$col = str_replace(' ', '_', $item['ColumnName']);
				if( ($item['RecordResponse']=='TRUE') && ($item['ItemType']!='Pick All That Apply Question') ) {
					if ( isset($item['AllowNullResponse']) && $item['AllowNullResponse']=='TRUE' ) {
						if( DEBUG_VERIFY ) {
							debugP($col . ' ALLOWS NULL RESPONSE');
						}	
					} elseif( $results[$col]==NULL ) {
						if( DEBUG_VERIFY ) {
							debugP($col . ' IS NULL');
						}
						$UnansweredQuestions[$col] = true;
						$verified = 0;
					} else {
						if( DEBUG_VERIFY ) {
							debugP($col . ' == ' . $results[$item['ColumnName']]);
						}
					}
				}
			}
		}
		return $verified;
	}
	
	function PromptUser($page) {
		global $form;
		global $results;
		
		if($page < 1) {
			return 0;
		} else {
			if( $form['prefs']['ForwardEnabledOnCompletion'] != 'TRUE' ){
				return 0;
			}
			$section = $form['sections'][$page - 1];
			$items = $section['items'];
			foreach($items as $item) {
				$col = str_replace(' ', '_', $item['ColumnName']);
				if( $item['RecordResponse']=='TRUE' ) {
					if ( !isset($results[$col]) && (!isset($item['AllowNullResponse']) || $item['AllowNullResponse']=='FALSE') ) {
						return 1;
					}
				}
			}
			return 0;
		}
	}
	
	################
	## OptionsArray
	################
	// Rows are delimited by chr(5)
	// Columns are delimited by chr(4)
	function OptionsArray($s) {
		$rows = split(chr(5), $s);
		for($i = 0; $i < count($rows); $i++){
			$rows[$i] = split(chr(4), $rows[$i]);
		}
		return $rows;
	}
	
	######################
	## RANDOM VARIABLES ##
	######################
	
	######################
	## SetRandomVariables
	######################
	
	function SetRandomVariables( $form ) {
		global $results;
		
		// $randomVariableSpecs = stringToArray($form['prefs']['RandomVariables'], chr(4), chr(5));
		$randomVariableSpecs = stringToArray($form['prefs']['RandomVariables'], chr(4), chr(5));
		
		if( DEBUG_RANDOMVARIABLES ) {
			debugP( 'Random Variable Specifications' , 'h3' );
			debugArray( $randomVariableSpecs );
		}
		
		foreach($randomVariableSpecs as $varname => $spec) {
			$spec = stringToArray($spec, ',', ';');
			$totalWeight = 0;
			foreach($spec as $weight) {
				$totalWeight += $weight;
			}
			// Multiply by 1000 to handle fractional weights
			$whichValue = mt_rand(1, $totalWeight * 1000);
			foreach($spec as $value => $weight) {
				$whichValue -= $weight * 1000;
				if( $whichValue <= 0 ) {
					$results[$varname] = $value;
					break;
				}
			}
		}
		
		if( DEBUG_RANDOMVARIABLES ) {
			debugP( 'Random Variable Settings' , 'h3' );
			debugArray( $results );
		}
		
	}
	
	##########################
	## CONDITIONAL SECTIONS ##
	##########################
	
	##################
	## IncludeSection
	##################
	function IncludeSection( $section ) {
		global $results;
		
		if(isset($section['Conditions'])){
			$conditions = stringToArray( $section['Conditions'], chr(4), chr(5) );
		} else {
			$conditions = array();
		}
		
		if( DEBUG_CONDITIONALSECTIONS ) {
			debugP( 'Section Conditions', 'h3' );
			debugP( $section['IncludeOrSkip'] . ' this section unless ' . $section['OneOrAll'] . ' of the conditions are met:' );
			debugArray( $conditions );
		}
		
		if( !isset($section['IncludeOrSkip'])){
			$Include = true;
		} else {
			if( $section['IncludeOrSkip'] == 'Include' )
				$Include = true;
			else
				$Include = false;
		}
		
		// If there are no conditions then we're done!
		if( count( $conditions ) == 0 ) {
			return $Include;
		}
			
		$OneConditionMet = false;
		$OneConditionFailed = false;
		
		foreach( $conditions as $varname => $condition ) {
			$condition = split( chr(4), $condition );
			$varValue = strtolower( $results[$varname] );
			$compareValue = strtolower( $condition[1] );
			switch($condition[0]) {
			case '=':
				if( $varValue == $compareValue )
					$OneConditionMet = true;
				else
					$OneConditionFailed = true;
				break;
			case '!=':
				if( $varValue != $compareValue )
					$OneConditionMet = true;
				else
					$OneConditionFailed = true;
				break;
			case '<':
				if( $varValue < $compareValue )
					$OneConditionMet = true;
				else
					$OneConditionFailed = true;
				break;
			case '>':
				if( $varValue > $compareValue )
					$OneConditionMet = true;
				else
					$OneConditionFailed = true;
				break;
			case '<=':
				if( $varValue <= $compareValue )
					$OneConditionMet = true;
				else
					$OneConditionFailed = true;
				break;
			case '>=':
				if( $varValue >= $compareValue )
					$OneConditionMet = true;
				else
					$OneConditionFailed = true;
				break;
			}
		}
		
		if( $section['OneOrAll'] == 'One' ) {
			if ( $OneConditionMet )
				$Include = !( $Include );
		}
		else {
			if ( !( $OneConditionFailed ) )
				$Include = !( $Include );
		}
		if( DEBUG_CONDITIONALSECTIONS ) {
			if( $Include )
				debugP( 'Section included');
			else
				debugP( 'Section skipped');
		}
		
		return $Include;
	}
	
	function NextPage($form, $page) {
		$pagecount = sectioncount();
		for( $p = $page + 1; $p <= $pagecount; $p++ ) {
			if( IncludeSection( $form['sections'][$p - 1] ) )
				return $p;
		}
		return $pagecount + 1;
	}
	
	function PreviousPage($form, $page) {
		for( $p = $page - 1; $p >= 1 ; $p-- ) {
			if( IncludeSection( $form['sections'][$p - 1] ) )
				return $p;
		}
		return 1;
	}
	
	## From PHP.NET
	
	function microtime_float(){
		list($usec, $sec) = explode(" ", microtime());
		return ((float)$usec + (float)$sec);
	}
	
	################
	## BuildSection
	################
	function BuildSection($form, $page, $HiliteUnanswered) {
		global $results;
		global $UnansweredQuestions;
		
		if( done() ){
			echoR('<p style="text-align: center; padding: 16px;">' . MESSAGE_THANKS . '</p>');
			
			if( REDIRECT_ON_COMPLETION ) {
				echoR( '<script language="javascript">' );
				echoR( 'location.replace("' . REDIRECT_ON_COMPLETION_URL . '")' );
				echoR( '</script>' );
				echoR( '<meta http-equiv="refresh" content="2;' . REDIRECT_ON_COMPLETION_URL . '">' );
				echoR( '<p align="center"><a href="' . REDIRECT_ON_COMPLETION_URL . '">' . REDIRECT_ON_COMPLETION_MESSAGE . '</a></p>' );
			}
		} else {
			$section = $form['sections'][$page - 1];
			
			$items = $section['items'];
			
			foreach($items as $item) {
				$type = $item['ItemType'];
				$col = str_replace(' ', '_', $item['ColumnName']);
				if( isset($UnansweredQuestions[$col]) && $UnansweredQuestions[$col] ) {	
					echoR( '<div class="item_hilited">' );
				} else {
					echoR( '<div class="item">' );
				}
				switch($type){
				// Text-Response Question
				case 'Text-Response Question':
					echoP( DecodeQuestion($item['QuestionText']) );
					echoR( '<blockquote>' );
					if ($item['Multiline'] == 'TRUE') {
						echoR ('<textarea name="COL_' . $col . '" cols="60">' . DecodeAnswer($results[$col]) . '</textarea>' );
					} else {
						echoR ('<input name="COL_' . $col . '" type="text" value="'. $results[$col] .'" size=60>' );
					}
					echoR( '</blockquote>' );
					break;
				// Multiple-Choice Question
				case 'Multiple-Choice Question':
					echoP( DecodeQuestion($item['QuestionText']) );
					$options = OptionsArray($item['Options']);
					echoR( '<blockquote><p>' );
					foreach($options as $option) {
						echo '<input name="COL_' . $col . '" type="radio" value="' . $option[1] .'" ';
						echo conditionalString($option[1] == $results[$col], 'checked') .'>';
						echo '&nbsp;' . DecodeQuestion( $option[0] );
						echoR( '<br>' );
					}
					echoR( '</p></blockquote>' );
					break;
				// Pick All That Apply Question
				case 'Pick All That Apply Question';
					echoP( DecodeQuestion($item['QuestionText']) );
					$options = OptionsArray($item['Options']);
					foreach($options as $option) {
						$qcol = $col . '_' .  $option[1];
						// Forces 0 value if the checkbox is not checked
						echo '<input type="HIDDEN" name="COL_' . $qcol . '" value="0">';
						echo '<input type="checkbox" name="COL_' . $qcol . '" value="1" ';
						echo conditionalString($results[$qcol] == '1', 'checked') .'>';
						echo '&nbsp;' . DecodeQuestion( $option[2] );
						echoR( '<br>' );
					}
				// Text Item
				case 'Text':
					echoP( DecodeQuestion($item['Text']) );
					break;
				// Range-Response Question
				case 'Range-Response Question':
					echoP( DecodeQuestion($item['QuestionText']) );
					echo '<p align="center">';
					echo "<table><tr>\n";
					echo '<td>' . $item['LabelMin'] . '&nbsp;&nbsp; </td><td>';
					for( $i = $item['ValueMin']; $i <= $item['ValueMax']; $i ++ ){
						echo '<div style="text-align: center;">&nbsp;<input name="COL_' . $col . '" type="radio" value="' . $i .'" ';
						if( isset($results[$col]) && ($i == $results[$col] ) ) {
							echo('checked');
						}
						echo '>&nbsp;';
						if( $form['prefs']['ShowRangeValues'] == 'TRUE' ){
							echo '<br>' . $i . '</div></td><td>';
						}
					}
					echo '&nbsp;' . $item['LabelMax'] . "</td>\n";
					echoP("</tr></table>");
					echoR( '</p>' );
					break;
				// Pick Image Question
				case 'Pick Image Question':
					echoP( DecodeQuestion($item['QuestionText']) );
					$options = OptionsArray($item['Options']);
					echoR( '<blockquote><p>' );
					echoR( '<table width="100%" border="0" cellspacing="0" cellpadding="4"><tr>' );
					foreach($options as $option) {
						echo '<p align="center"><td><img alt=" " src="' . IMAGE_PATH . $option[0] . '">';
						echo '<br>';
						echo '<input name="COL_' . $col . '" type="radio" value="' . $option[1] .'" ';
						echo conditionalString($option[1] == $results[$col], 'checked') .'>';
						echoR( '&nbsp;' . DecodeQuestion( $option[2] ).'</td></p>' );
					}
					echoR( '<tr></table>' );
					echoR( '</p></blockquote>' );
					break;
				// Picture
				case 'Picture':
					echoP( '<img alt=" " src="' . IMAGE_PATH . $item['picture'] . '">' );
					break;
				// Video
				case 'Video':
					echo '<p><embed src="' . VIDEO_PATH . $item['video'] . '"';
					if( isset($item['Width']) ){
						echo ' width="' . $item['Width'] . '"';
					} else {
						echo ' width="640"';
					}
					if( isset($item['Height']) ){
						$h = $item['Height'];
					} else {
						$h = 480;
					}
					if( $item['ShowControls'] == 'TRUE' ){
						$h += 16;
					}
					echo ' height="' . $h . '"';
					echo ' SCALE="Aspect"';
					echo ' AUTOPLAY="' . $item['PlayAuto'] . '"';
					echo ' LOOP="' . $item['Loop'] . '"';
					echo ' CONTROLLER="' . $item['ShowControls'] . '"';
					echoR( ' pluginspage="http://quicktime.apple.com/"></embed></p>' );
					break;
				default:
					echoP('Error in form. Unknown item type: ' . $type, 'h3');
				}
				if( DEBUG_ITEMS ) {
					debugArray( $item );
				}
				echoR( '</div>' );
			}
		}
	}
	
	// umask(077);
	$form = loadForm();
	
	// Initialise results (prevent warning!) 10/30/03
	$results = array();
	loadSettings($form);

	$HiliteUnanswered = 0;
	if($_SERVER['REQUEST_METHOD'] == 'GET') {
		// If no session exists, create one
		$SessionID = date('Y_m_d_H_i_s_') . mt_rand(100000,999999);
		SetRandomVariables( $form );
		//$CurrentPageNumber = 1;
		$CurrentPageNumber = NextPage( $form, 0 );
		
		// Section Timer Support
		// Record when we started section
		$results['_timebuilt'] = microtime_float();
	} elseif ($_SERVER['REQUEST_METHOD'] == 'POST') {
		// Recover session state
		$SessionID = rawp('SessionFile');
		// Load existing session data
		$CurrentPageNumber = rawp("CurrentPageNumber");
		
		$SubmitButton = rawp('SubmitButton');
		UpdateSession($SessionID);
	
		// Section Timer Support
		// Record time spent in previous section
		$s = $form['sections'][$CurrentPageNumber - 1];
		$t = microtime_float();
		if( isset($s['TimerEnabled']) && isset($s['TimerColumnName']) && $s['TimerEnabled'] == 'TRUE' ){
			$results[$s['TimerColumnName']] = $t - floatVal($results['_timebuilt']);
		}
		// Record when we started section
		$results['_timebuilt'] = $t;
		
		if($SubmitButton == BUTTON_CONTINUE_NAME ) {
			// If the relevant questions were answered
			$UnansweredQuestions = array();
			if( VerifyAnswers($CurrentPageNumber) ) {
				// $CurrentPageNumber += 1;
				$CurrentPageNumber = NextPage( $form, $CurrentPageNumber );
				$HiliteUnanswered = 0;
			} else {
			// otherwise hilite unanswered questions
				$HiliteUnanswered = 1;
			}
		} elseif ($SubmitButton == ' Back ') {
			// $CurrentPageNumber -= 1;
			$CurrentPageNumber = PreviousPage( $form, $CurrentPageNumber );
		}
	} else {
		die('This page only supports GET and POST requests.');
	}
	SaveSession($SessionID);
			
	if( ($form['prefs']['ShowSection'] == 'TRUE') && $CurrentPageNumber && !Done() ) {
		$section_name = $form['sections'][$CurrentPageNumber - 1]['SectionName'];
		$form_title = FORM_NAME . ' &gt; ' . $section_name;
	} else {
		$section_name = '';
		$form_title = FORM_NAME;
	}
?>
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">
<!--    This page was generated by RiddleMeThis Runtime   -->
<!--               http://riddlemethis.net/               -->
<!-- Written by and copyright (c) 2003-2008 Tonio Loewald -->
<html>
<head>
<title>
<?php echo $form_title ?>
</title>
<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1">
<style type="text/css">
	* {
		margin: 0;
	}
	
	a:link {
		text-decoration: none;
		color: #30F;
	}
	
	a:visited {
		text-decoration: none;
		color: #008;
	}
	
	a:hover {
		text-decoration: none;
		color: #30F;
		background-color: #DDD;
	}
	
	a:active {
		text-decoration: none;
		color: #000;
		background-color: #FFF;
	}
	
	html {
		height: 100%;
	}
	
	body {
<?php
	if( !done() ){
		$section = $form['sections'][$CurrentPageNumber - 1];
		if ( isset( $section['BackgroundPicture'] ) ){
			echo 'background-image: url("'. IMAGE_PATH . $section['BackgroundPicture'] .'");' . "\n";
			$bg_picture = 1;
		}
	}
?>
		height: 100%;
		background-repeat : no-repeat;
		background-position: center top;
		background-color : #<?php echo COLOR_BACKGROUND ?>;
		color : #<?php echo COLOR_TEXT ?>;
		font-family : <?php echo FONT_OPTIONS ?>;
		font-size : <?php echo FONT_SIZE ?>px;
	}
	
	.normal {
		background-color : #<?php echo COLOR_BACKGROUND ?>;
	}
	
	.alert {
		border: 4px solid #<?php echo COLOR_ALERT ?>;
	}
	
	.item {
		width : auto;
		padding : 8px 40px 7px 40px;
		border-color: #<?php echo COLOR_HEADER ?>;
		border-width: 0px 0px 1px 0px;
		border-style: dotted;
	}
	
	.item_hilited {
		width : auto;
		padding : 6px 36px 6px 36px;
		border-color: #<?php echo COLOR_ALERT ?>;
		border-width: 2px 4px 2px 4px;
		border-style: solid;
	}

	#main {
		left: 0px;
		right: 0px;
		height: auto;
		width: auto;
	}
	
	table, td, th {
		font-family : <?php echo FONT_OPTIONS ?>;
		font-size : <?php echo FONT_SIZE ?>px;
	}
	
	#header {
		top : 0px;
		left : 0px;
		right : 0px;
		
		text-align : center;
		text-shadow: rgba(0,0,0,0.4) 1px 2px 1px;
		background-color : #<?php echo COLOR_HEADER ?>;
/*
		filter: alpha(opacity=80);
		opacity : 0.8;
*/
	}
	
	.title {
		font-weight: bold;
		text-shadow: rgba(0,0,0,0.4) 1px 2px 1px;
	}
	
	.frame {
		text-align : center;
<?php
	if( $bg_picture ){
		// nothing
?>
<?php
	} else {
?>
		background-color : #<?php echo COLOR_HEADER ?>;
		filter: alpha(opacity=80);
		opacity : 0.8;
<?php
	}
?>
	}
	
	#footer {
		text-align: center;
		background-color : #<?php echo COLOR_HEADER ?>;
		text-shadow: rgba(0,0,0,0.4) 1px 2px 1px;
		left: 0px;
		right: 0px;
		bottom: 0px;
		padding: 2px;
		height: auto;
		font-family : Verdana, Sans-Serif;
		font-size : 10px;
	}
	
	#debug {
		margin : 10px;
		font-family : Verdana, Sans-Serif;
		font-size : 10px;
	}
</style>
</head>
<body>
	<div id="header">
		<p style="padding: 8px;">
			<?php
				$prefs = $form['prefs'];
				if( $section_name != '' ){
					echo '&nbsp;<span class="title">&nbsp;' . $section_name . '&nbsp;</span>&nbsp;';
				}
				if( $HiliteUnanswered ) {
					echo HEADING_ANSWERHILITED;
				} else {
					if( done() ) {
						echo HEADING_DONE;
					} elseif( PromptUser($CurrentPageNumber) ) {
						echo HEADING_ANSWERALL;
					} else {
						echo '&nbsp;';
					}
				}
			?>
		</p>
	</div>
	<div id="main">
		<form method="POST" action="<?php echo $_SERVER['PHP_SELF'] ?>">
			<input name="SessionFile" type="hidden" value=<?php echo $SessionID; ?>>
			<input name="CurrentPageNumber" type="hidden" value=<?php echo $CurrentPageNumber; ?>>
			<?php
				BuildSection($form, $CurrentPageNumber, $HiliteUnanswered);
							
				// General Debugging Information
				if( DEBUG_FORMPREFS ) {
					debugP( 'Form Preferences' , 'h3' );
					debugArray( $form['prefs'] );
				}
				
				if( DEBUG_VERIFY ) {
					debugP( 'UnansweredQuestions', 'h3' );
					debugArray( $UnansweredQuestions );
				}
				
				if( DEBUG_RESULTS ) {
					debugP( 'Results', 'h3' );
					debugArray( $results );
					debugP( 'Post Variables', 'h3' );
					listvars();
				}
				
				if( DEBUG_SERVER ) {
					debugP( 'Server', 'h3' );
					debugArray( $_SERVER );
				}
				
				if( $debugOutput != '' ){
					echoR('<div id="debug">');
					echo $debugOutput;
					echoR('</div>');
				}
			?>
			<table width="100%">
				<tr>
					<td style="width: 20%; padding: 4px;">
						<?php
							// Back Button
							if( $prefs['BackButtonEnabled'] == 'TRUE' ) {
								if( ($CurrentPageNumber > 1) && (! done())) {
									echo '<input name="SubmitButton" type="submit" value=" ' . BUTTON_BACK_NAME . ' ">';
								} else {
									echo '&nbsp;';
								}
							}
						?>
					</td>
					<td style="width: 60%; text-align: center; padding: 4px;">
						<?php 
							// Show Progress
							if( ($prefs['ShowProgress'] == 'TRUE') && ($CurrentPageNumber) && (!done()) ){
								echoR('Page ' . $CurrentPageNumber . '/' . sectioncount());
							} else {
								echoR('&nbsp;');
							}
						?>
					</td>
					<td style="width: 20%; text-align: right; padding: 4px;">
						<?php
							// Continue Button
							if( ! done() ) {
								echo '<input name="SubmitButton" type="submit" value="' . BUTTON_CONTINUE_NAME . '">';
							} else {
								echo '&nbsp;';
							}
						?>
					</td>
				</tr>
			</table>
		</form>
	</div>
<?php	if( !done() ){ ?>
	<div id="footer">
		<?php echo( FORM_FOOTER ); ?>
	</div>
<?php	} ?>	
</body>
</html>
